package org.ow2.jonas.admin.auditconsole.service;

import java.io.Serializable;

import org.ow2.util.auditreport.impl.GenericAuditReport;
import org.ow2.util.auditreport.impl.InvocationAuditReport;
import org.ow2.util.auditreport.impl.JNDIAuditReport;
import org.ow2.util.auditreport.impl.WebInvocationAuditReport;

/**
 * @author Mathieu Ancelin
 */
public class HealthService implements Serializable {

    /**
     * Serial Version UID.
     */
    private static final long serialVersionUID = 8145014260425696138L;

    private long globalCounter = 0;

    private double globalMeanTime = 0;

    private double gobalTotalTime = 0;

    private long ejbCounter = 0;

    private double ejbMeanTime = 0;

    private double ejbTotalTime = 0;

    private long webCounter = 0;

    private double webMeanTime = 0;

    private double webTotalTime = 0;

    private long jndiCounter = 0;

    private double jndiMeanTime = 0;

    private double jndiTotalTime = 0;

    private double actualTrend = 0;

    private double previousTrend = 0;

    private double actualDurationTrend = 0;

    private double previousDurationTrend = 0;

    private double actualEJBTrend = 0;

    private double previousEJBTrend = 0;

    private double actualEJBDurationTrend = 0;

    private double previousEJBDurationTrend = 0;

    private double actualWEBTrend = 0;

    private double previousWEBTrend = 0;

    private double actualWEBDurationTrend = 0;

    private double previousWEBDurationTrend = 0;

    private double actualJNDITrend = 0;

    private double previousJNDITrend = 0;

    private double actualJNDIDurationTrend = 0;

    private double previousJNDIDurationTrend = 0;

    private long trendTimestamp = 0;

    public void addReport(final GenericAuditReport report) {
        long actualTimestamp = System.currentTimeMillis();
        if (actualTimestamp > (trendTimestamp + 30000)) {
            previousTrend = actualTrend;
            previousDurationTrend = actualDurationTrend;
            previousEJBTrend = actualEJBTrend;
            previousEJBDurationTrend = actualEJBDurationTrend;
            previousWEBTrend = actualWEBTrend;
            previousWEBDurationTrend = actualWEBDurationTrend;
            actualJNDITrend = previousJNDITrend;
            trendTimestamp = actualTimestamp;
            actualTrend = 0;
            actualEJBTrend = 0;
            actualWEBTrend = 0;
            actualJNDITrend = 0;
        }
        if (report instanceof InvocationAuditReport) {
            InvocationAuditReport actualReport = ((InvocationAuditReport) report);
            ejbCounter++;
            gobalTotalTime += actualReport.getRequestDuration();
            ejbTotalTime += actualReport.getRequestDuration();
            if (ejbCounter > 0) {
                ejbMeanTime = ejbTotalTime / ejbCounter;
            }
            actualEJBDurationTrend = ((ejbTotalTime * ejbMeanTime) + actualReport.getRequestDuration()) / (ejbTotalTime + 1);
            actualDurationTrend = ((gobalTotalTime * globalMeanTime) + actualReport.getRequestDuration())
                / (gobalTotalTime + 1);
        } else if (report instanceof WebInvocationAuditReport) {
            WebInvocationAuditReport actualReport = ((WebInvocationAuditReport) report);
            webCounter++;
            gobalTotalTime += actualReport.getRequestDuration();
            webTotalTime += actualReport.getRequestDuration();
            if (webCounter > 0) {
                webMeanTime = webTotalTime / webCounter;
            }
            actualWEBDurationTrend = ((webTotalTime * webMeanTime) + actualReport.getRequestDuration()) / (webTotalTime + 1);
            actualDurationTrend = ((gobalTotalTime * globalMeanTime) + actualReport.getRequestDuration())
                / (gobalTotalTime + 1);
        } else if (report instanceof JNDIAuditReport) {
            JNDIAuditReport actualReport = ((JNDIAuditReport) report);
            jndiCounter++;
            gobalTotalTime += actualReport.getRequestDuration();
            jndiTotalTime += actualReport.getRequestDuration();
            if (jndiCounter > 0) {
                jndiMeanTime = jndiTotalTime / jndiCounter;
            }
            actualJNDIDurationTrend = ((jndiTotalTime * jndiMeanTime) + actualReport.getRequestDuration())
                / (jndiTotalTime + 1);
            actualDurationTrend = ((gobalTotalTime * globalMeanTime) + actualReport.getRequestDuration())
                / (gobalTotalTime + 1);
        }
        globalCounter++;
        if (globalCounter > 0) {
            globalMeanTime = gobalTotalTime / globalCounter;
        }
    }

    public ServerHealth getServerHealth() {
        ServerHealth health = new ServerHealth();
        health.setGlobalCounter(globalCounter);
        health.setGlobalMeanTime(globalMeanTime);
        health.setGobalTotalTime(gobalTotalTime);
        health.setEjbCounter(ejbCounter);
        health.setEjbMeanTime(ejbMeanTime);
        health.setEjbTotalTime(ejbTotalTime);
        health.setWebCounter(webCounter);
        health.setWebMeanTime(globalMeanTime);
        health.setWebTotalTime(webTotalTime);
        health.setJndiCounter(jndiCounter);
        health.setJndiMeanTime(jndiMeanTime);
        health.setJndiTotalTime(jndiTotalTime);
        if (((int) actualTrend) > ((int) previousTrend)) {
            health.setGlobalTrend("up");
        } else if (((int) actualTrend) < ((int) previousTrend)) {
            health.setGlobalTrend("down");
        }
        if (((int) actualDurationTrend) > ((int) previousDurationTrend)) {
            health.setGlobalDurationTrend("up");
        } else if (((int) actualDurationTrend) > ((int) previousDurationTrend)) {
            health.setGlobalDurationTrend("down");
        }
        if (((int) actualEJBTrend) > ((int) previousEJBTrend)) {
            health.setEJBTrend("up");
        } else if (((int) actualEJBTrend) < ((int) previousEJBTrend)) {
            health.setEJBTrend("down");
        }
        if (((int) actualEJBDurationTrend) > ((int) previousEJBDurationTrend)) {
            health.setEJBDurationTrend("up");
        } else if (((int) actualEJBDurationTrend) < ((int) previousEJBDurationTrend)) {
            health.setEJBDurationTrend("down");
        }
        if (((int) actualWEBTrend) > ((int) previousWEBTrend)) {
            health.setWEBTrend("up");
        } else if (((int) actualWEBTrend) < ((int) previousWEBTrend)) {
            health.setWEBTrend("down");
        }
        if (((int) actualWEBDurationTrend) > ((int) previousWEBDurationTrend)) {
            health.setWEBDurationTrend("up");
        } else if (((int) actualWEBDurationTrend) < ((int) previousWEBDurationTrend)) {
            health.setWEBDurationTrend("down");
        }
        if (((int) actualJNDITrend) > ((int) previousJNDITrend)) {
            health.setJNDITrend("up");
        } else if (((int) actualJNDITrend) < ((int) previousJNDITrend)) {
            health.setJNDITrend("down");
        }
        if (((int) actualJNDIDurationTrend) > ((int) previousJNDIDurationTrend)) {
            health.setJNDIDurationTrend("up");
        } else if (((int) actualJNDIDurationTrend) < ((int) previousJNDIDurationTrend)) {
            health.setJNDIDurationTrend("down");
        }
        return health;
    }
}
